/*
    libgpmi - General Package/Module Interface
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef GPMI_MODULES_H
#define GPMI_MODULES_H
/* This file is the module init helper file. Include it from main.c in a module.
   Mainly helper macros are here to provide dynamic/static function names
	 compile time. */

#include <gpmi/mods-static.h>

/* Require  module name */
#if (gpmi_mod_name == 0x1234)
	#error "You have to define MOD_NAME in Makefile.am"
#endif

#define str_macro_(nm) #nm
#define str_macro(nm) str_macro_(nm)


/* Function name generator macros */
#ifndef gpmi_mod_static
	#define gpmi_mod(function)            gpmi_mod_ ## function
#else
	#define gpmi_mod__(function, modname) gpmi_mod_ ## function ## _ ## modname
	#define gpmi_mod_(function, modname)  gpmi_mod__(function, modname)
	#define gpmi_mod(function)            gpmi_mod_(function, gpmi_mod_name)
#endif

/* These are the entry points the module must define */
void gpmi_mod(RunTick)     (gpmi_module *module, int frame_counter);
void gpmi_mod(RunData)     (gpmi_module *module, void *data);
int  gpmi_mod(Debug)       (gpmi_module *module, const char *parameters);
int  gpmi_mod(Initialize)  (gpmi_module *module, const char *params);
void gpmi_mod(Uninitialize)(gpmi_module *module);

/* Static init routines */
#ifdef gpmi_mod_static

void gpmi_mod(static_reg_module)(gpmi_module *module)
{
	module->proc_Init     = gpmi_mod(Initialize);
	module->proc_UnInit   = gpmi_mod(Uninitialize);
	module->proc_RunTick  = gpmi_mod(RunTick);
	module->proc_RunData  = gpmi_mod(RunData);
	module->proc_Debug    = gpmi_mod(Debug);
}

void gpmi_mod(static_construct)(void) __attribute__ ((constructor)) ;

void gpmi_mod(static_construct)(void)
{
	gpmi_reg_static_module(str_macro(gpmi_mod_name), (void *)gpmi_mod(static_reg_module));
}

void gpmi_mod(static_destruct)(void) __attribute__ ((destructor)) ;

void gpmi_mod(static_destruct)(void)
{
	gpmi_unreg_static_module(str_macro(gpmi_mod_name));
}

#endif /* gpmi_mod_static */

#endif /* GPMI_MODULES_H */
