/*
    libgpmi - General Package/Module Interface - example module
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "gpmi.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "../../packages/example.h"
#include <gpmi/modules.h>
#include <gpmi/threads.h>

/* Before we migrate to dl-compat, for dlls, we need extern, since there's no LAZY there. */
gpmi_keyword *Example_colors_kw;

/* This is an example module for gpmi */

typedef struct example_data {
	char *params;
} example_data;

void event_handler_global(gpmi_module *mod, int event, ...) {
	printf("Example-mod: catched event %d [general]\n", event);
}

void event_handler(gpmi_module *mod, int event, int p1, int p2) {
	printf("Example-mod: catched event %d [by event ID] [ p1= %d == 2005 ]  [ p2= %d == 1 ]\n", event, p1, p2);
}

char *PkgW_Example_color_rot(int argc, char *argv[]);
char *PkgW_Example_color_inv(int argc, char *argv[]);

gpmi_package *example_mod_pkg;

static void print_error(gpmi_err_stack_t *entry, char *string)
{
	printf("> %s\n", string);
}

int gpmi_mod(Initialize)(gpmi_module *module, const char *params)
{
	example_data *data;
	int ev_num;
	char *ev_types, *tmp;
	gpmi_keyword *kw;
	int clr, err_stack_save;
	char *color_arg[] = {
		"PkgW_Example_color_rot",
		"green"
	} ;
	char *color_arg2[] = {
		"PkgW_Example_color_rot",
		"BLACK"
	} ;
	char *color_arg3[] = {
		"PkgW_Example_color_rot",
		"red+blue"
	} ;


	printf("Example-mod: Module init with params '%s'\n",params);

	/* Store local module-specific data */
	data = malloc(sizeof(example_data));
	data->params = strdup(params);

	err_stack_save = gpmi_err_stack_enable();

	/* Now we open a package - without this, we wouldn't have events */
	if (gpmi_pkg_load("example", 0, NULL, NULL, &example_mod_pkg) != 0) {
		printf("Example-mod: couldn't load package example\n");
		gpmi_err_stack_process_str(print_error);
	}
	else {
		if (err_stack_save == 0)
			gpmi_err_stack_disable();

		/* Resolve an event and compare it to the exported version */
		ev_num =  gpmi_event_find("Example_nothing", &ev_types);
		printf("Example-mod: Found event Example_nothing as %d == %d , types: '%s'\n", ev_num, Example_nothing, ev_types);

		/* Bind event Example_nothing for both global and per ID */
		gpmi_event_bind_global(module, event_handler_global);
		gpmi_event_bind(module, Example_nothing, event_handler);

		/* Make the package generate the event */
		Example_do_nothing(1);

		/* We resolve example package's keywords and compare them to the c exports from the package */
		{
			kw = gpmi_keyword_find_ptr(gpmi_shared.gpmi_keywords, "Example_colors");
			printf("Example-mod: colors: " PRINTF_PTR " == " PRINTF_PTR "\n", PRINTF_PTR_CAST Example_colors_kw, PRINTF_PTR_CAST kw);
			clr = gpmi_keyword_find_int(Example_colors_kw, "green");
			printf("Example-mod: green: %d == %d\n", example_green, clr);
		}

		/* Test string based calling against integer keyword array */
		{
			/* Expect: blue */
			printf("Example-mod: Color-rot= %s == blue\n", tmp=PkgW_Example_color_rot(2, color_arg)); free(tmp);
			/* Expect: error message and NULL */
			tmp = PkgW_Example_color_rot(2, color_arg2);
			printf("Example-mod: Color-rot= " PRINTF_PTR " == " PRINTF_PTR " \n", PRINTF_PTR_CAST tmp, PRINTF_PTR_CAST NULL);
		}

		/* Test string based calling against multiple integer keyword array */
		{
			/* Expect: red+blue */
			printf("Example-mod: Color-inv= %s == red+blue\n", tmp=PkgW_Example_color_inv(2, color_arg)); free(tmp);
			/* Expect: green */
			printf("Example-mod: Color-inv= %s == green\n", tmp=PkgW_Example_color_inv(2, color_arg3)); free(tmp);
		}
	}


	printf("\n");
	module->mod_data = data;
	return 0;
}

void gpmi_mod(Uninitialize)(gpmi_module *module)
{
	example_data *data;

	if (example_mod_pkg != NULL)
		gpmi_pkg_unload(example_mod_pkg);

	data = module->mod_data;
	printf("Example-mod: Module uninit (module init params were %s)\n", data->params);

	free(data->params);
	free(data);
}

void gpmi_mod(RunTick)(gpmi_module *module, int frame_counter)
{
	printf("Example-mod: RunTick, frame %d\n", frame_counter);
}

void gpmi_mod(RunData)(gpmi_module *module, void *data)
{
	printf("Example-mod: RunData, data " PRINTF_PTR " ; gpmi_current_module is " PRINTF_PTR " != " PRINTF_PTR "\n", PRINTF_PTR_CAST data, PRINTF_PTR_CAST gpmi_get_current_module(), PRINTF_PTR_CAST NULL);
}

int gpmi_mod(Debug)(gpmi_module *module, const char *command)
{
	printf("Example-mod: got command: '%s'\n", command);

	return 0;
}
