#include <stdio.h>
#include "gpmi.h"
#include "common.h"
#include "timer.h"

char *(*t_timer_init)(int argc, char *argv[]) = NULL;
char *(*t_timer_uninit)(int argc, char *argv[]) = NULL;
char *(*t_timer_run)(int argc, char *argv[]) = NULL;
char *(*t_timer_add)(int argc, char *argv[]) = NULL;

timer_list *(*b_timer_init)(timer_callback *CallBackProc) = NULL;
void (*b_timer_uninit)(timer_list *TimerList) = NULL;
void (*b_timer_run)(timer_list *TimerList, int frame_counter) = NULL;
void (*b_timer_add)(timer_list *TimerList, int frmae_counter, int TTL, int Frequency, void *Data) = NULL;

pkg_proc_t pkg_procs[] = {
	{(void **)(void *)&t_timer_init,		(void **)(void *)&b_timer_init,			"timer_init"},
	{(void **)(void *)&t_timer_uninit,	(void **)(void *)&b_timer_uninit,		"timer_uninit"},
	{(void **)(void *)&t_timer_run,			(void **)(void *)&b_timer_run,			"timer_run"},
	{(void **)(void *)&t_timer_add,			(void **)(void *)&b_timer_add,			"timer_add"},
	{NULL,											NULL, 											NULL}
} ;

int ran[4];
int id[4] = {0, 1, 2, 3};



void timer_cb(int cmd, void *data, timer *timer)
{
	int tmr;

	tmr = *(int *)data;
	switch(cmd) {
		case TIMER_INIT:
			ran[tmr] = 0;
			break;
		case TIMER_TICK:
			ran[tmr]++;
			break;
	}
}

void print_error(gpmi_err_stack_t *entry, char *string)
{
	printf("  %s\n", string);
}

int main()
{
	int fail = 0;
	timer_list *tl;
	gpmi_package *pkg_timer;
	int n;

	/* Load the package */
	init();

	/* Enable detailed error reporting */
	gpmi_err_stack_enable();

	/* Load the timer package */
	if ((n = gpmi_pkg_load("timer", 0, reg_cmd, "MyHandle", &pkg_timer)) != 0) {
		fprintf(stderr, "timer: Failed to load timer package. Details: %d\n", n);
		gpmi_err_stack_process_str(print_error);

		return 1;
	}

	/* First we check if all necessary functions are exported */
	chk_cmd();

	/* From now on, we use the binary interface */
	tl = b_timer_init((timer_callback *)timer_cb);
	if (tl == NULL) {
		fprintf(stderr, "timer: Failed to init a timer list!\n");
		return 1;
	}

	/* Init test timers */
	b_timer_add(tl, 0, 5, 1, (void *)&id[1]);
	b_timer_add(tl, 0, 0, 2, (void *)&id[2]);
	b_timer_add(tl, 0, 0, 3, (void *)&id[3]);

	/* Run 8 ticks */
	for(n=1; n<9; n++)
		b_timer_run(tl, n);

	/* TTL limited timer 1 to run only 5 times */
	if (ran[1] != 5) {
		fprintf(stderr, "TTL didn't work: %d\n", ran[1]);
		fail++;
	}

	/* 8/2 is 4 */
	if (ran[2] != 4) {
		fprintf(stderr, "timer 2 failed: %d\n", ran[2]);
		fail++;
	}

	/* 8/3 is 2 */
	if (ran[3] != 2) {
		fprintf(stderr, "timer 3 failed: %d\n", ran[3]);
		fail++;
	}

	/* Uninitialize everything */
	b_timer_uninit(tl);
	gpmi_pkg_unload(pkg_timer);
	gpmi_uninit();

	printf("Test finished.\n");

	return fail;
}
