/*
    libgpmi - General Package/Module Interface - wrapgen
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "wrapgen.h"
#include "generators.h"
#include "../gpmi/global.h"

/* All the files */
static str_buff_t	*wrp_enums_c      = NULL,
									*wrp_enums_list   = NULL,
									*wrp_enums_uninit = NULL;


void GenPkgEnumsFiles(int command)
{
	FILE *f;
	switch (command) {
		case open_files:
			wrp_enums_c      = str_buff_open();
			wrp_enums_list   = str_buff_open();
			wrp_enums_uninit = str_buff_open();
			sbprintf(wrp_enums_c,       "void %s%s_enum_init() {\n", PREFIX_GPMI_PKG_GEN, name_space);
			sbprintf(wrp_enums_list,    "\n");
			sbprintf(wrp_enums_uninit,  "void %s%s_enum_uninit() {\n", PREFIX_GPMI_PKG_GEN, name_space);

			break;
		case close_files:
			sbprintf(wrp_enums_c,      "}\n");
			sbprintf(wrp_enums_list,   "\n");
			sbprintf(wrp_enums_uninit, "}\n");

			cfopen(f, gen_prefix "enums.c", "w");
			str_buff_print(f, wrp_enums_list);
			str_buff_print(f, wrp_enums_c);
			str_buff_print(f, wrp_enums_uninit);
			cfclose(f);

			str_buff_close(wrp_enums_c);
			str_buff_close(wrp_enums_list);
			str_buff_close(wrp_enums_uninit);
			break;
	}
}


/* Wrapper generator for enums */
void GenPkgEnums(void *current, int line, char *cmd, char *cmd_type, int cmd_type_flags, int params)
{
	char buff[1024];
	int n;

	/* Empty input line */
	if ( cmd == NULL ) {
		sbprintf(wrp_enums_c, "\n");
		return;
	}

	/* Generate spaces */
	for(n = 0; n <= 256; n++)
		buff[n] = ' ';
	buff[n] = 0;

	sbprintf(wrp_enums_list, "gpmi_keyword *kw_%s = NULL;\n", cmd);

	sbprintf(wrp_enums_list, "static const gpmi_keyword_static_int kws_%s[] = {\n", cmd);
	for(n = 0; n < params; n++) {
		sbprintf(wrp_enums_list, "\t{ \"%s\", %s },\n", param[n], param[n]);
	}

	sbprintf(wrp_enums_list,   "\t{ NULL, 0 }\n};\n");
	sbprintf(wrp_enums_c,      "\tkw_%s = gpmi_keyword_import(\"%s\",   gpmi_keyword_type_integer, kws_%s);\n", cmd, cmd, cmd);

	sbprintf(wrp_enums_uninit, "\tif (kw_%s != NULL) {\n", cmd);
	sbprintf(wrp_enums_uninit, "\t\tgpmi_keyword_destroy(kw_%s);\n", cmd);
	sbprintf(wrp_enums_uninit, "\t\tkw_%s = NULL;\n\t}\n", cmd);
}
