/*
    libgpmi - General Package/Module Interface - listgen
    Copyright (C) 2005-2007 Patric 'TrueLight' Stout & Tibor 'Igor2' Palinkas
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

%{
	#include <stdlib.h>
	#include <stdio.h>
	#include <string.h>
	#include "listgen1.h"
%}

%union {
	int num;
	char *str;
}

%token	<str>	ID
%token	<num> NUMBER
%token	<str>	ENUM
%token	<str>	CMT_OPEN
%token	<str>	CMT_CLOSE
%token	<str>	UNSIGNED
%token	<str>	INT
%token	<str>	DOUBLE
%token	<str>	CHAR
%token	<str>	VOID
%token	<str> EVENT
%token	<str> TYPEDEF
%token	<str>	GPMI_KEYWORD
%token	<str>	GPMI_FAKE_KEYWORD
%token	<str>	GPMI_DEFINE_EVENT
%token	<str>	OF
%token	<str>	MULTIPLE
%token	<str>	DYNAMIC
%token	<str>	CONST
%token	<str>	STRUCT
%token	<str>	FAKETYPE
%token	<str>	EXTERN
%token	<str> UNION
%token	<str> DIRECT
%token	<str>	NOWRAP
%token  <str> INLINE

%type		<str> type
%type		<str> type_
%type		<str> type_standard
%type		<str> enum_list
%type		<str> enum_element

%locations
%error-verbose

%%


headers:
	/* empty */
	| headers header
	;

header:
					enum
				|	function
				| faketype
				| event
				| keyword_def
				| typedef
				| CMT_OPEN junks CMT_CLOSE		/* ignore comments */
				| struct
				| vardef
				| union
/* Too dangerous to enable this: */
/*				| junks ';'										/* ignore statements we can't handle */
				;

union:
		UNION '{' headers '}' id_list_member ';'
	| TYPEDEF UNION ID '{' headers '}' id_list_member ';'
	;

typedef:
		TYPEDEF type ID ';'				{faketype_add($3, $2);}
	| TYPEDEF type ID '(' { curr_params = NULL; } param_list_or_empty ')' ';'
	| TYPEDEF type '(' ptr ID ')' '(' { curr_params = NULL; } param_list_or_empty ')' ';'
	| TYPEDEF type '(' ID ')' '(' { curr_params = NULL; } param_list_or_empty ')' ';'

	| NOWRAP TYPEDEF type ID '(' { curr_params = NULL; } param_list_or_empty ')' ';' { fprintf(stderr, "Warning: you shouldn't use nowrap before a function typedef as a function typedef won't have a wrapper anyway (function '%s').\n", $4); }

	;

faketype:
	FAKETYPE '(' type_ ',' ID	')' ';' { faketype_add($5, $3); }
	;

junks:
				/* empty */
			| junk junks
			;

junk:
				ID
			| ENUM
			| EVENT
			| TYPEDEF
			| GPMI_KEYWORD
			| GPMI_DEFINE_EVENT
			| EXTERN
			| OF
			| '/'
			| '|'
			| ':'
			| ';'
			| '{'
			| '}'
			| ','
			| '('
			| ')'
			| '.'
			| '!'
			| '<'
			| '>'
			| '['
			| ']'
			| '='
			| UNION
			| MULTIPLE
			| NUMBER
			| STRUCT
			| INT
			| UNSIGNED
			| DOUBLE
			| VOID
			| CONST
			| CHAR
			| DYNAMIC
			| DIRECT
			| NOWRAP
			| FAKETYPE
			| '*'
			;
			
enum:
			TYPEDEF ENUM ID '{' 												{ verbprintf(1, "FOUND: enum %s {\n", $3); enum_create($3); }
													enum_list '}' ID ';'		{ verbprintf(1, " } -> %s\n", $8); faketype_add($8, "int"); enum_close(); }
		|	NOWRAP TYPEDEF ENUM ID '{' 												{ verbprintf(1, "FOUND: enum %s {\n", $3); enum_nowrap = 1;}
													enum_list '}' ID ';'		{ verbprintf(1, " } -> %s\n", $9); enum_nowrap = 0;}

		;

enum_list:
						/* empty */	{}
					|	enum_list enum_element { enum_add_element($2); }
					;

enum_element:
								ID ','								{ verbprintf(2, "    %s, \n", $1); $$ = $1;}
							| ID '=' expr ','				{ verbprintf(2, "    %s = expr, \n", $1); $$ = $1 }
							| CMT_OPEN junks CMT_CLOSE	{ $$ = NULL; }
							;

expr:
		/* empty */
	| '*' expr
	| '/' expr
	| '+' expr
	| '-' expr
	| '|' expr
	| '&' expr
	| '=' '=' expr
	| '!' '=' expr
	| '<' expr
	| '>' expr
	| '(' expr
	| ')' expr
	| NUMBER expr
	| ID expr
	;

function:
				type ID '(' 																					{ 
																															verbprintf(1, "FOUND: function %s:%s; params: ", $2, $1);
																															func_add($1, $2);
																														}
									param_list_or_empty ')' ';'								{ verbprintf(1, "\n"); curr_params = NULL; }

			| type '(' ptr ID ')' '(' 													{ 
																															verbprintf(1, "FOUND: function (* %s):%s; params: ", $4, $1);
																															func_add($1, $4);
																														}
									param_list_or_empty ')' ';'								{ verbprintf(1, "\n"); curr_params = NULL; } 

			| DIRECT CHAR '*' ID '(' INT ID ',' CHAR '*' '*' ID ')' ';'      { func_add_direct($4); }
			| DIRECT CHAR '*' ID '(' INT ID ',' CHAR '*' ID '[' ']' ')' ';'  { func_add_direct($4); }
			| NOWRAP type ID '(' param_list_or_empty ')' ';'          			 { /* Don't generate wrapper for this */ curr_params = NULL;}
			| NOWRAP type '(' ptr ID ')' '(' param_list_or_empty ')' ';'     { /* Don't generate wrapper for this */ curr_params = NULL;}
			| NOWRAP INLINE type ID '(' param_list_or_empty ')' ';'          			 { /* Don't generate wrapper for this */ curr_params = NULL;}
			| NOWRAP INLINE type '(' ptr ID ')' '(' param_list_or_empty ')' ';'     { /* Don't generate wrapper for this */ curr_params = NULL;}

			;

event:
			GPMI_DEFINE_EVENT '(' ID  ')' '(' 										{ verbprintf(1, "FOUND: event %s; params: ", $3); event_add($3); }
																		param_list ')' ';'			{ verbprintf(1, "\n"); }
			;

param_list_or_empty_drop:
	{curr_params_push(); curr_params = NULL; } param_list_or_empty {curr_params_pop();}
	;
		
param_list_or_empty:
			/* empty */
		| param_list
		| VOID
		;

param_list:
							type ID										{ verbprintf(2, "%s:%s ", $2, $1); param_add($1, $2); }
						|	type ID ',' param_list		{ verbprintf(2, "%s:%s ", $2, $1); param_add($1, $2); }
						| '.' '.' '.'								{ verbprintf(2, "... "); }
						| type '(' ptr ID ')'	'('  param_list_or_empty_drop ')'  { verbprintf(2, "%s:void* ", $4); param_add(strdup("void*"), $4); }
						| type '(' ptr ID ')' '('  param_list_or_empty_drop ')' ',' param_list  { verbprintf(2, "%s:void* ", $4); param_add(strdup("void*"), $4); }
						;


type:
			DYNAMIC type_		{ $$ = chr_insert($2, '!'); }
		|	CONST type_			{ $$ = chr_insert($2, '^'); }
		|	type_						{ $$ = $1; }
		;
		

type_:
				type_standard														{ $$ = malloc(128); strcpy($$, $1); }
			| INT CMT_OPEN ID CMT_CLOSE								{ $$ = malloc(128); sprintf($$, "int/%s", $3); }
			| UNSIGNED INT CMT_OPEN ID CMT_CLOSE			{ $$ = malloc(128); sprintf($$, "uint/%s", $4); }
			| UNSIGNED CHAR CMT_OPEN ID CMT_CLOSE			{ $$ = malloc(128); sprintf($$, "uchar/%s", $4); }
			| ID																			{ check_kw($1); $$ = malloc(128); sprintf($$, "%s", $1); }
			| ID ptr																	{ check_kw($1); $$ = malloc(128); sprintf($$, "ptr"); }
			| MULTIPLE ID															{ check_kw($2); $$ = malloc(128); sprintf($$, "@%s", $2); }
			;

type_standard:
				INT								{ $$ = "int"; }
			| INT ptr						{ $$ = "int*"; }
			|	UNSIGNED INT			{ $$ = "uint"; }
			|	UNSIGNED INT ptr	{ $$ = "uint*"; }
			| DOUBLE						{ $$ = "double"; }
			| DOUBLE ptr				{ $$ = "double*"; }
			| CHAR 							{ $$ = "char"; }
			| CHAR ptr					{ $$ = "char*"; }
			| UNSIGNED CHAR			{ $$ = "uchar"; }
			| UNSIGNED CHAR ptr	{ $$ = "uchar*"; }
			| VOID							{ $$ = "void"; }
			| VOID ptr					{ $$ = "void*"; }
			;

ptr:
		'*'
	| ptr '*'
	;


keyword_def:
				GPMI_KEYWORD '*' ID ';' CMT_OPEN OF ID CMT_CLOSE { verbprintf(1, "FOUND: keyword definition %s of %s\n", $3, $7); keyword_add($3, kw_type_export, $7); }
			|	EXTERN GPMI_KEYWORD '*' ID ';' CMT_OPEN OF ID CMT_CLOSE { verbprintf(1, "FOUND: keyword definition %s of %s\n", $4, $8); keyword_add($4, kw_type_export, $8); }
			|	GPMI_FAKE_KEYWORD '(' '*' ID ')' ';' { verbprintf(1, "FOUND: keyword definition (local) %s\n", $4); keyword_add($4, kw_type_export, NULL);}
			;

/* These are known but ignored structures */

vardef:
			type id_list ';'
		| EXTERN vardef
		| EXTERN type_ '(' ptr id_list_member ')' '(' {curr_params = NULL;} param_list_or_empty ')' ';'
		;
		
id_list:
			id_list_member
		|	id_list_member ',' id_list
		;

id_list_member:
		 	ID
		| ID '[' ']'
		| ID '[' ID ']'
		| ID '[' NUMBER ']'
		| ID ':' NUMBER
		;

struct:
				TYPEDEF STRUCT ID '{' {in_struct = 1;}	headers '}' ID ';'  {in_struct = 0;}
			|	STRUCT ID '{' {in_struct = 1;} headers '}' ';'  {in_struct = 0;}
			| TYPEDEF STRUCT ID ID ';' 
		;
